-------------------------------------------------------------------------------
-- Title      : EB675001DIP User CPLD template
-------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity template is
    Port ( --# CPU interface
	 		  nrst_i : in std_logic;								--# reset
           cko_i : in std_logic;									--# clock out from Oki
           nxcs_i : in std_logic_vector(3 downto 2);		--# I/O bank chip select
           nxoe_i : in std_logic;								--# output enable / read enable
           nxwe_i : in std_logic;								--# write enable
           nubs_i : in std_logic;                        --# upper byte select
           nlbs_i : in std_logic;                        --# lower byte select
           xwait_o : out std_logic;								--# external I/O bank wait signal 
           xd : inout std_logic_vector(15 downto 0);		--# data bus
           xa_i : in std_logic_vector(23 downto 21);		--# upper address bus to dicern I/O access or MEM access
           xa0_i : in std_logic;								--# Lower address bus to dicern I/O access or MEM access
			  --# Dedicated pins
			  ncplden_i : in std_logic;							--# cpld enable from ctrl cpld
			  nbufen_io : inout std_logic;						--# buffer enable (address bus)
			  nwait_i : in std_logic;								--# wait pin from davicom (network)

			  -- signals to quad uart chip

           iord : out std_logic;
           iowr : out std_logic;
           irq_out : out std_logic;
           clk_out : out std_logic;
           rst_out : out std_logic;
           
			  led  : out std_logic_vector(3 downto 0);
           en   : inout std_logic_vector(3 downto 0);
			  irq  : in std_logic_vector(3 downto 0);
			  ucs  : out std_logic_vector(3 downto 0);
           
           ext_data : inout std_logic_vector(7 downto 0);
			  
			  d_29 : out std_logic
		  );
end template;

architecture Arch of template is


----------------------------------------------------------------------------
-- DO NOT REMOVE
----------------------------------------------------------------------------
	signal user_wait_n : std_logic;		-- Internal wait user pin, active low
	signal nbufen : std_logic;				-- Internal buffer enable, active low

	signal user_bus : std_logic_vector(7 downto 0);	-- internal bus
	signal irq_en	 : std_logic_vector(3 downto 0); -- irq enable control
	signal irq_state: std_logic_vector(3 downto 0);
	signal led_state: std_logic_vector(3 downto 0);   
	signal en_state : std_logic_vector(3 downto 0);
	signal en_ctrl  : std_logic_vector(3 downto 0);

	signal uart_clk : std_logic;

	signal clk_counter : std_logic_vector(2 downto 0);	
begin

----------------------------------------------------------------------------
--				!!!			OPTIONAL LOGIC			!!!
----------------------------------------------------------------------------
--
-- nbufen_io :	Use only if the design has to output the address bus to 
--					the outside world, enable address buffers
--
----------------------------------------------------------------------------
	
	nbufen_io <= nbufen when nbufen='0' else 'Z';
	nbufen <= nxcs_i(3) and ncplden_i;

	-- Drive the data bus to cpu (read command)
	--xd(7 downto 0) <= user_bus_out(7 downto 0) when nxoe_i='0' and ncs='0' and nlbs_i='0' else (others=>'Z')  ;
	--xd(15 downto 8) <= user_bus_out(15 downto 8) when nxoe_i='0' and ncs='0' and nubs_i='0' else (others=>'Z')  ;
 	--ncs <= cs_n_i(2) and cs_n_i(3) and  ncplden_i;

	-- copy sa0 through when chip-select is enabled

	d_29 <= xa0_i when nxcs_i(3)='0' else '0';

	-- using cs2/cs3
	--
	-- the default configuration for these using the ABLE bootloader is 16bit
	-- which necessitates the use of nubs_i/nlbs_i to determine which data
	-- path the data is being presented on
	--
	-- 8bit access to 16bit area (even address)
	--		xa0_i 				= '0'
	--		nlbs_i				= '0'
	--		xd(7 downto 0)		= data
 	--
	-- 8bit access to 16bit area (odd address)
	--		xa0_i 				= '1'
	--		nubs_i				= '0'
	--		xd(15 downto 8)	= data
 	--		

	-- data bus buffering for read
	--
	--	  when nxcs_i(2) (cs2) is active, pass the internal bus out to the cpu
	--	  when nxcs_i(3) (cs3) is active, pass the low end of connector d to cpu bus

	xd(7 downto 0) <= user_bus(7 downto 0) when (nxoe_i='0' and nxcs_i(2)='0') else
							ext_data when (nxoe_i='0' and nxcs_i(3)='0' and nlbs_i='0') else
							(others => 'Z');

	xd(15 downto 8) <= ext_data when (nxoe_i='0' and nxcs_i(3)='0' and nubs_i='0') else
							 (others => 'Z');

	-- data bus buffering for write
	--
	--	  when nxcs_i(3) (cs3) is active, pass cpu data bus out to low end of connector d

  	ext_data <= xd(7 downto 0) when (nxwe_i='0' and nxcs_i(3)= '0' and nlbs_i='0') else
							xd(15 downto 8) when (nxwe_i='0' and nxcs_i(3)= '0' and nubs_i='0') else
							(others => 'Z');

   -- led state output

	led <= led_state(3 downto 0);

	-- enable control

	en(0) <= 'Z' when en_ctrl(0)='0' else en_state(0);
	en(1) <= 'Z' when en_ctrl(1)='0' else en_state(1);
	en(2) <= 'Z' when en_ctrl(2)='0' else en_state(2);
	en(3) <= 'Z' when en_ctrl(3)='0' else en_state(3);

   -- drive uart select signals onto the following
	--
	--	c(3 downto 6) are uart select 0,1,2,3 (active low)
	
	ucs(0)	<= '0' when xa_i = "000" and nxcs_i(3)='0' else '1';	
	ucs(1)	<= '0' when xa_i = "001" and nxcs_i(3)='0' else '1';	
	ucs(2)	<= '0' when xa_i = "010" and nxcs_i(3)='0' else '1';	
	ucs(3)	<= '0' when xa_i = "011" and nxcs_i(3)='0' else '1';	

	-- use the next pins as iord/iowr  (active low)
	--

	iord	<=	'0' when nxoe_i='0' and nxcs_i(3)='0' else '1';
	iowr  <= '0' when nxwe_i='0' and nxcs_i(3)='0' else '1';

	-- put the baud-rate clock out of pin c(9)

	clk_out  <= uart_clk;

	-- interrupt sampling
	--
	--	pins d(11 to 14) are irqs 0 through 3.

	irq_state(0) <= irq(0);
	irq_state(1) <= irq(1);
	irq_state(2) <= irq(2);
	irq_state(3) <= irq(3);

	-- combined interrupt out

	irq_out	<=	'0' when (irq_state and irq_en) /= "00000000" else '1'; 

	-- reset output

	rst_out <= '1' when nrst_i='0' else '0';

----------------------------------------------------------------------------
--	!!!			CHANGE ONLY IF YOU KNOW WHAT YOU ARE DOING			!!!
----------------------------------------------------------------------------
--
--	user_wait_n : 	If not used must be set to one otherwise wait pin will be
--						active all the time holding the cpu indefinetly
--	nwait_i 		: 	Is the network controller wait signal and has to be routed
--						to the cpu for correct operation
--
----------------------------------------------------------------------------
 	xwait_o <= '1' when nwait_i='0' or user_wait_n='0' else '0';
	user_wait_n <= '1';	


----------------------------------------------------------------------------
--	!!!			USER DESIGN			!!!
----------------------------------------------------------------------------

	internal_regs_wr : process(nxcs_i, nxwe_i, xa_i, xd, nrst_i) 
	begin
	   if (nrst_i='0') then
			irq_en <= X"0";	-- disable IRQs at startup
		else
		if (nxcs_i(2)='0') then
			-- latch registers in at the end of the cycle

			if (nxwe_i'event and nxwe_i='1') then
				case xa_i is
					when "001" =>
						irq_en <= xd(3 downto 0);
					when "100" =>
						led_state <= xd(3 downto 0);
					when "110" =>
						en_state <= xd(3 downto 0);
						en_ctrl  <= xd(7 downto 4);
					when others =>
						-- nothing here at the moment
				end case;
			end if;
		end if;
		end if;		
	end process;

	internal_regs_rd : process(nxcs_i, nxoe_i, xa_i)
	begin
		if (nrst_i='1' and nxcs_i(2)='0') then
			if (nxoe_i'event and nxoe_i='0') then
				case xa_i is
					when "000" =>
						user_bus <= X"A0";
					when "001" =>
						user_bus <= "0000" & irq_en;
					when "010" =>
						user_bus <= "0000" & irq_state;
					when "011" =>
						user_bus <= "0000" & (irq_state and irq_en);
					when "100" =>
						user_bus <= "0000" & led_state;
					when "101" =>
						user_bus <= "0000" & en;
					when "110" =>
						user_bus <= en_ctrl & en_state;
					when others =>
						user_bus <= X"00";
				end case;
			end if;
		end if;
	end process;

  	clk_divisor : process(cko_i)
	begin
		if (cko_i'event and cko_i = '1') then
			clk_counter <= clk_counter + 1;
			uart_clk <= clk_counter(2);		-- divide by 8
		end if;
	end process;
end Arch;
